# Styles


We use [stylus](http://stylus-lang.com) because we can omit semi-colons and use some neat functions, e.g. string interpolation for calc, [lighten](http://stylus-lang.com/docs/bifs.html#lightencolor-amount), darken, ...

css file = `.styl` file

## Usage

So we use separated `.styl` files across the app and we let webpack bundle them to one css file via the `ExtractTextPlugin`

A special webpack loader `stylus-loader` translates the `.styl` file syntax into real css syntax in memory

## Global styles

There are some global styles that are mean to be used across the app, these are located in `frontend/src/styles/`

- `behaviours.styl` contains styles for behaviors [TODO rename to behavior...]
- `common.styl` contains variables and rules for various things
- `fixes.styl` contains rules that should fix some other rule (e.g. in some browsers)
- `styles.ts` contains variables (should have the same value as in `common.styl`) so that they can be accessed by js

## Local styles

Every component can have it's own styles, they should be located as close as possible to the component

If the component (parent component) has sub component which use the same style this is also ok, then the style file is located next to the parent component


## Why no css modules [info]

[react css modules](https://github.com/gajus/react-css-modules)

We first started this but the autocompletion is too bad and it feels bad somehow and it cannot be debugged well
In the browser you cannot inspect (or change?) the rules if running the web dev server


## Adding style files

If you want to add rules to an existing file there is nothing you need to take care of (only don't use rule names twice as always)

[We first started by a basic approach every style file imported the variable file if needed
This leaded to a css file where the variables where copied for every import
Because the variables are inside teh `common.styl` file (which is the biggest css file) we got a really large css file
To fix this we declared a file `frontend/src/allStyles.styl` that contains references to all `.styl` files and thus all files are connected --> can use the variables and no rules were copied or duplicated
Because webpack only respect/include files that were required somehow we required the `allStyles.styl` file in `frontend/src/index.tsx`, the root component
]

- To add a new css file (lets say the name is `my.styl`) create the file & add your rules
- open the file `frontend/src/allStyles.styl` and add a new statement importing your file

```styl
@import "path/to/my.styl"
```

**Please make sure the order of the files is kept in sync with the file tree view (so if you expand the file tree the order from top to down should be the same in the file tree and the `allStyles.styl` file)**

