# Install and run the ClientServer (which is the main backend)

## Prerequisites

* Having installed [git](https://git-scm.com/) (This is for cloning the repository only.)
* Having installed [Dotnet SDK 1.1.11](https://gitlab.informatik.uni-halle.de/Syndrom/Docs/-/blob/extended-guides/installation/dependencies/dotnet.md)
* Having installed [PostgreSQL](https://gitlab.informatik.uni-halle.de/Syndrom/Docs/-/blob/extended-guides/installation/dependencies/postgresql.md)
* Having updated the config file for PostgreSQL [(short guide)](https://gitlab.informatik.uni-halle.de/Syndrom/Docs/-/blob/extended-guides/installation/yapexDb.md#postgresql-settings)
* Having created the PostgreSQL user "yapex" and the database "yapexDb" [(short guide)](https://gitlab.informatik.uni-halle.de/Syndrom/Docs/-/blob/extended-guides/installation/yapexDb.md#creating-and-configuring-yapexdb)
* Having created a build of [frontend-react](https://gitlab.informatik.uni-halle.de/Syndrom/Docs/-/blob/extended-guides/installation/frontend-react.md)

This guide is tested in Debian 9 (Stretch)

## 1. Choose at which directory the project should be

In production (or when you try to emulate a project-alike environment) you should always use `/opt/yapex/ClientServer`. While developing you can use any use any directory you like.

## 2. Clone the repository

Clone the repository `https://gitlab.informatik.uni-halle.de/Syndrom/ClientServer.git` to the directory you chose.

## 3. Choose a branch (and checkout it)

While developing use `develop`. For running without errors (especially in production) use `master`.
```bash
git checkout [branch]
```

## 4. Change the environment variables according to your system

The environment variables live in `ClientServer/src/ClientServer/ClientServer.csproj`.
Change

```xml
<RuntimeFrameworkVersion>1.1.11</RuntimeFrameworkVersion>
<RuntimeIdentifiers>debian.9-x64</RuntimeIdentifiers>
```

`1.1.11` is the installed RUNTIME (not SDK!) version of Dotnet. (For getting the installed runtime version use the command `dotnet --info`. It's the version of the `Microsoft.NETCore.App`.)
`Debian` is the OS name.
`9` is the major OS version.
`x64` is the system architecture which is most common (64-bit architecture) for servers. (For example `ARM` would be used for most mobile devices and `i386` for most older 32-bit architecture systems.)


## 5. Create appsettings.json
Copy (or rename) the file `ClientServer/src/ClientServer/appsettings_example.json` to `ClientServer/src/ClientServer/appsettings.json`.

Open that file. Within "DbConnectionString", change username to `yapex` and password to the password you gave to the postgreSql user `yapex`.

And set "IsDbBootstrapEnabled" to `true`. (Otherwise the database tables aren't created automatically.)

## 6. Copy the build frontend files

(You find those at `frontend-react/dist/` within the `frontend-react` project.)

Copy those to `ClientServer/src/ClientServer/wwwroot/`.


## 7. Install Dotnet Dependencies and build

Move to `ClientServer/src/ClientServer`. Run

```bash
dotnet restore
dotnet ef database update
```
Please note: when you first start the server you might see `system settings not found, cannot run submissions scheduler.NextRun` in the logs
this is because the scheduler (which runs every X mins) tries to access the system settings (first row in table in the db) but we have no data yet.

The first command installs the dependencies. The second runs all current database migrations. This happens automatically if `"IsDbBootstrapEnabled"` within `appsettings.json` is set to `true`. While this is True, migrations can't be undone! For undoing migrations, set it to `false` and then run `dotnet ef migrations remove`.


## 8. Run the server

There are several ways to do this.

a) in `ClientServer/src/ClientServer/`: run `dotnet run`

b) in `ClientServer/src/ClientServer`: run `dotnet build`. This prints the path to where a file called `ClientServer.dll` is created. That can be ran with `dotnet <path_to>ClientServer.dll`

c) publish a build to `output/`. in `ClientServer/src/ClientServer/`: run `dotnet publish --configuration Release --framework netcoreapp1.0  -o output/ClientServer`. Now, in `ClientServer/src/ClientServer/output/` there is a file `ClientServer.dll`. Move to that directory and run `dotnet ClientServer.dll`


## 9. Open the website in a browser

Open `http://localhost:5000/api/init` the first time the project is run before opening anything else! (Otherwise, you can completely reinstall it – or find a way to undo what you just did and tell me how). If there is a JSON-Error, you can safely ignore it. By the way, that page is supposed to be empty. Nothing wrong here.

Now, open `http://localhost:5000/index_deploy.html#/`. The admin-login is `username: j, pwd: j`.

